#!/usr/bin/perl

# perl script to modify a Warplan scenario file (.scn),
# changing the weather probabilities
#
# The basic concept is to set each zone-month's weather to be
# 100% chance of the most common weather according to the
# current weather probabilities. In addition, the script
# over-rides some weather, especially for
# the Arctic Cicle, North Marine, and Arctic (Russia) weather zones.

use Getopt::Std;

getopts('vx');

$opt_v || $opt_x or die "usage: withvec.pl {-v|-x} filename.scn {byte offset before beginning of weather data}
  -v read file parse and display
  -x execute: modify data and put result in filename.scn.new
";

my $f = shift @ARGV;
my $location = shift @ARGV || 390700;
my $numbytes =  200100100; # infinite

open(IN,$f) or die "Could not open $f";

our $OFFSET=$location;

our $buffer = undef;
sysread(IN,$buffer,100100100);
close(IN);


my @monthnames = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;

my $wbyj = { 0 => 'clear', # this is not really true. Clear is what you get if no other chances happen
	     5 => 'rain',
	     9 => 'heavyrain',
	     13 => 'snow',
	     17 => 'blizzard',
};

# 5 = rain
# 9 = heavy rain
# 13 = snow
# 17 = blizzard

# this is the hash of overrides.
# $byzone->{$zone}->{$month} = number for weather to be set to 100%

my $byzone = { 1 => { 3 => 5, # bug where April was messed up
	       },
	       # zone 5 north marine goes all the way to Murmansk
	       # Lots of heavy rain both because the North Atlantic is very
	       # story but does not freeze and because it is hard
	       # to find enemy ships in only a few hours of daylight
	       5 => { 0 => 9, 
		      1 => 9,
		      2 => 5,
		      3 => 5,
		      9 => 5,
		      10 => 9,
		      11 => 9,
	       },
	       # 9 arctic circle
	       9 => { 4=>9,
		      9=>9,
	       },
	       # 10 arctic, includes Leningrad and the Baltic States
	       # Summer campaign season is only June-Sep
	       # April & May are Rasputitsa
	       10 => { 9 => 5, # october rain
		       10 => 9, # nov heavy rain
		       11 => 13, # dec snow
		       0 => 17, # jan blizzard
		       1 => 17, # feb blizzard
		       2 => 17, # march blizzard
		       3 => 9, # april heavy rain
		       4 => 9, # may heavy rain
	       },
};

my $printcount = 0;
my $month = 0;
my $zone = 0;

my $i = 0;
while( $i++ < $numbytes ) {
    if ( (getb($i) == 221 || getb($i) == 220) &&
	 getb($i+1) == 254 &&
	 getb($i+2) == 255 &&
	 getb($i+3) == 121 &&
	 getb($i+4) == 221 &&
	 getb($i+5) == 254 &&
	 getb($i+6) == 255) {
	$printcount = 27;
	if ($month++ == 11) {
	    $zone++;
	    print "\n===== zone $zone =======";
	    $month = 0;
	    last if $zone > 11;
	}
	if ($opt_v) {
	    print "\n ",$monthnames[$month],"  ";
	} else {
	    printf("\n%6d %3d ",$i,getb($i-1));
	}
	
	$OFFSET = $OFFSET+$i+6;
	$i=1;
	my $maxj = 0;
	my $max = 0;
	my $sum = 0;
	foreach my $j (1..20) {
	    my $v = getb($j);
	    if ($v > 0) {
		if ($opt_v) {
		    # display mode
		    my $weather = $wbyj->{$j};
		    print "$weather:$v ";
		}
		next unless $opt_x;
		$sum += $v;
		if ($v > $max) {
		    $max = $v;
		    $maxj= $j;
		}
		# clear this value
		vec($buffer,$OFFSET+$j,8)=0;
	    }
	}
	next unless $opt_x;
	# got to the end. If we found a max, set that column to 100
	print "maxj:$maxj max:$max sum:$sum";
	my $override = $byzone->{$zone}->{$month};
	if ( $override ) {
	    print " OVERRIDE: $override";
	    vec($buffer,$OFFSET+$override,8)=100;
	} elsif ($max > 0 && $sum > 49) {
	    print " SET";
	    vec($buffer,$OFFSET+$maxj,8)=100;
	}
    }
}

print "\n";

exit unless $opt_x;

my $fn = $f . ".new";
open(OUT,'>',$fn) or die "Could not open $fn";
syswrite(OUT,$buffer);

sub getb {
    my $loc = shift;
    my $realloc = $OFFSET + $loc;
    return vec($buffer,$realloc,8);
}
